@@ -7,13 +7,11 @@ pub mod metadata;
 pub mod model;
 pub mod motion_photo;
 pub mod repo;
-pub mod scanner;
-pub mod thumbnail;
+pub mod thumbnailer;
 
 pub use model::PictureId;
 
 pub use model::Metadata;
 pub use motion_photo::MotionPhotoExtractor;
 pub use repo::Repository;
-pub use scanner::Scanner;
-pub use thumbnail::Thumbnailer;
+pub use thumbnailer::PhotoThumbnailer;
