@@ -11,15 +11,18 @@ use fotema_core::PictureId;
 use fotema_core::database;
 use fotema_core::people;
 use fotema_core::photo;
+use fotema_core::thumbnailify::Thumbnailer;
 use fotema_core::video;
 use fotema_core::visual;
+use fotema_core::Scanner;
+use fotema_core::FlatpakPathBuf;
+use fotema_core::people::migrate::Migrate;
 
 use std::result::Result::Ok;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, Mutex};
 
 use std::collections::VecDeque;
-use std::path::PathBuf;
 use std::time::Instant;
 
 use tracing::{error, info, warn};
@@ -27,21 +30,28 @@ use tracing::{error, info, warn};
 use anyhow;
 
 use super::{
-    load_library::{LoadLibrary, LoadLibraryInput, LoadLibraryOutput},
-    photo_clean::{PhotoClean, PhotoCleanInput, PhotoCleanOutput},
-    photo_detect_faces::{PhotoDetectFaces, PhotoDetectFacesInput, PhotoDetectFacesOutput},
-    photo_enrich::{PhotoEnrich, PhotoEnrichInput, PhotoEnrichOutput},
-    photo_extract_motion::{PhotoExtractMotion, PhotoExtractMotionInput, PhotoExtractMotionOutput},
-    photo_recognize_faces::{
-        PhotoRecognizeFaces, PhotoRecognizeFacesInput, PhotoRecognizeFacesOutput,
+    load_library_task::{LoadLibraryTask, LoadLibraryTaskInput, LoadLibraryTaskOutput},
+    library_scan_task::{LibraryScanTask, LibraryScanTaskInput, LibraryScanTaskOutput},
+    person_thumbnail_task::{PersonThumbnailTask, PersonThumbnailTaskInput, PersonThumbnailTaskOutput},
+    photo_clean_task::{PhotoCleanTask, PhotoCleanTaskInput, PhotoCleanTaskOutput},
+    photo_detect_faces_task::{
+        PhotoDetectFacesTask, PhotoDetectFacesTaskInput, PhotoDetectFacesTaskOutput,
     },
-    photo_scan::{PhotoScan, PhotoScanInput, PhotoScanOutput},
-    photo_thumbnail::{PhotoThumbnail, PhotoThumbnailInput, PhotoThumbnailOutput},
-    video_clean::{VideoClean, VideoCleanInput, VideoCleanOutput},
-    video_enrich::{VideoEnrich, VideoEnrichInput, VideoEnrichOutput},
-    video_scan::{VideoScan, VideoScanInput, VideoScanOutput},
-    video_thumbnail::{VideoThumbnail, VideoThumbnailInput, VideoThumbnailOutput},
-    video_transcode::{VideoTranscode, VideoTranscodeInput, VideoTranscodeOutput},
+    photo_enrich_task::{PhotoEnrichTask, PhotoEnrichTaskInput, PhotoEnrichTaskOutput},
+    photo_extract_motion_task::{
+        PhotoExtractMotionTask, PhotoExtractMotionTaskInput, PhotoExtractMotionTaskOutput,
+    },
+    photo_recognize_faces_task::{
+        PhotoRecognizeFacesTask, PhotoRecognizeFacesTaskInput, PhotoRecognizeFacesTaskOutput,
+    },
+    photo_thumbnail_task::{PhotoThumbnailTask, PhotoThumbnailTaskInput, PhotoThumbnailTaskOutput},
+    video_clean_task::{VideoCleanTask, VideoCleanTaskInput, VideoCleanTaskOutput},
+    video_enrich_task::{VideoEnrichTask, VideoEnrichTaskInput, VideoEnrichTaskOutput},
+    video_thumbnail_task::{VideoThumbnailTask, VideoThumbnailTaskInput, VideoThumbnailTaskOutput},
+    video_transcode_task::{VideoTranscodeTask, VideoTranscodeTaskInput, VideoTranscodeTaskOutput},
+
+    tidy_task::{TidyTask, TidyTaskInput, TidyTaskOutput},
+    migrate_task::{MigrateTask, MigrateTaskInput, MigrateTaskOutput},
 };
 
 use crate::app::FaceDetectionMode;
@@ -57,25 +67,34 @@ pub enum MediaType {
     Video,
 }
 
+#[derive(Debug)]
+pub enum ThumbnailType {
+    Photo,
+    Video,
+    Face,
+}
+
 /// FIXME very similar (but different) to progress_monitor::TaskName.
 /// Any thoughts about this fact?
 #[derive(Debug)]
 pub enum TaskName {
     LoadLibrary,
-    Scan(MediaType),
+    Scan,
     Enrich(MediaType),
     MotionPhoto,
-    Thumbnail(MediaType),
+    Thumbnail(ThumbnailType),
     Clean(MediaType),
     DetectFaces,
     RecognizeFaces,
     Transcode,
+    Tidy,
+    Migrate,
 }
 
 #[derive(Debug)]
 pub enum BootstrapInput {
-    /// Configure the pictures library root
-    Configure(PathBuf),
+    /// Configure the pictures library root and host path
+    Configure(FlatpakPathBuf),
 
     /// Settings updated
     SettingsUpdated(Settings),
@@ -93,9 +112,12 @@ pub enum BootstrapInput {
     ScanPictureForFaces(PictureId),
     ScanPicturesForFaces,
 
-    // Queue task for transcoding videos
+    /// Queue task for transcoding videos
     TranscodeAll,
 
+    /// Queue task for processing motion photos
+    ProcessMotionPhotos,
+
     /// A background task has started.
     TaskStarted(TaskName),
 
@@ -127,34 +149,34 @@ pub struct Controllers {
 
     settings_state: SettingsState,
 
-    //pic_base_dir: Option<PathBuf>,
-
     // Stop background tasks.
     stop: Arc<AtomicBool>,
 
     /// Whether a background task has updated some library state and the library should be reloaded.
     library_stale: Arc<AtomicBool>,
 
-    load_library: Arc<WorkerController<LoadLibrary>>,
+    load_library_task: Arc<WorkerController<LoadLibraryTask>>,
+    library_scan_task: Arc<WorkerController<LibraryScanTask>>,
 
-    photo_scan: Arc<WorkerController<PhotoScan>>,
-    video_scan: Arc<WorkerController<VideoScan>>,
+    photo_enrich_task: Arc<WorkerController<PhotoEnrichTask>>,
+    video_enrich_task: Arc<WorkerController<VideoEnrichTask>>,
 
-    photo_enrich: Arc<WorkerController<PhotoEnrich>>,
-    video_enrich: Arc<WorkerController<VideoEnrich>>,
+    photo_clean_task: Arc<WorkerController<PhotoCleanTask>>,
+    video_clean_task: Arc<WorkerController<VideoCleanTask>>,
 
-    photo_clean: Arc<WorkerController<PhotoClean>>,
-    video_clean: Arc<WorkerController<VideoClean>>,
+    photo_thumbnail_task: Arc<WorkerController<PhotoThumbnailTask>>,
+    video_thumbnail_task: Arc<WorkerController<VideoThumbnailTask>>,
 
-    photo_thumbnail: Arc<WorkerController<PhotoThumbnail>>,
-    video_thumbnail: Arc<WorkerController<VideoThumbnail>>,
+    photo_extract_motion_task: Arc<WorkerController<PhotoExtractMotionTask>>,
 
-    photo_extract_motion: Arc<WorkerController<PhotoExtractMotion>>,
+    photo_detect_faces_task: Arc<WorkerController<PhotoDetectFacesTask>>,
+    photo_recognize_faces_task: Arc<WorkerController<PhotoRecognizeFacesTask>>,
 
-    photo_detect_faces: Arc<WorkerController<PhotoDetectFaces>>,
-    photo_recognize_faces: Arc<WorkerController<PhotoRecognizeFaces>>,
+    video_transcode_task: Arc<WorkerController<VideoTranscodeTask>>,
 
-    video_transcode: Arc<WorkerController<VideoTranscode>>,
+    tidy_task: Arc<WorkerController<TidyTask>>,
+    migrate_task: Arc<WorkerController<MigrateTask>>,
+    person_thumbnail_task: Arc<WorkerController<PersonThumbnailTask>>,
 
     /// Pending ordered tasks to process
     /// Wow... figuring out a type signature that would compile was a nightmare.
@@ -200,6 +222,11 @@ impl Controllers {
                 self.add_task_video_transcode();
                 self.run_if_idle();
             }
+            BootstrapInput::ProcessMotionPhotos => {
+                info!("Queueing task to process motion photos");
+                self.add_task_photo_extract_motion();
+                self.run_if_idle();
+            }
             BootstrapInput::TaskStarted(task_name) => {
                 info!("Task started: {:?}", task_name);
                 let _ = sender.output(BootstrapOutput::TaskStarted(task_name));
@@ -248,111 +275,136 @@ impl Controllers {
         };
     }
 
-    fn add_task_photo_scan(&mut self) {
-        let sender = self.photo_scan.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(PhotoScanInput::Start)));
-    }
-
-    fn add_task_video_scan(&mut self) {
-        let sender = self.video_scan.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(VideoScanInput::Start)));
+    fn add_task_library_scan(&mut self) {
+        let sender = self.library_scan_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(LibraryScanTaskInput::Start)));
     }
 
     fn add_task_photo_enrich(&mut self) {
-        let sender = self.photo_enrich.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(PhotoEnrichInput::Start)));
+        let sender = self.photo_enrich_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(PhotoEnrichTaskInput::Start)));
     }
 
     fn add_task_video_enrich(&mut self) {
-        let sender = self.video_enrich.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(VideoEnrichInput::Start)));
+        let sender = self.video_enrich_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(VideoEnrichTaskInput::Start)));
     }
 
     fn add_task_photo_thumbnail(&mut self) {
-        let sender = self.photo_thumbnail.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(PhotoThumbnailInput::Start)));
-    }
-
-    fn add_task_video_thumbnail(&mut self) {
-        let sender = self.video_thumbnail.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(VideoThumbnailInput::Start)));
-    }
-
-    fn add_task_photo_clean(&mut self) {
-        let sender = self.photo_clean.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(PhotoCleanInput::Start)));
-    }
-
-    fn add_task_video_clean(&mut self) {
-        let sender = self.video_clean.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(VideoCleanInput::Start)));
-    }
-
-    fn add_task_photo_extract_motion(&mut self) {
-        let sender = self.photo_extract_motion.sender().clone();
+        let sender = self.photo_thumbnail_task.sender().clone();
         self.enqueue(Box::new(move || {
-            sender.emit(PhotoExtractMotionInput::Start)
+            sender.emit(PhotoThumbnailTaskInput::Start)
         }));
     }
 
+    fn add_task_video_thumbnail(&mut self) {
+        let sender = self.video_thumbnail_task.sender().clone();
+        self.enqueue(Box::new(move || {
+            sender.emit(VideoThumbnailTaskInput::Start)
+        }));
+    }
+
+    fn add_task_photo_clean(&mut self) {
+        let sender = self.photo_clean_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(PhotoCleanTaskInput::Start)));
+    }
+
+    fn add_task_video_clean(&mut self) {
+        let sender = self.video_clean_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(VideoCleanTaskInput::Start)));
+    }
+
+    fn add_task_photo_extract_motion(&mut self) {
+        let sender = self.photo_extract_motion_task.sender().clone();
+        let enable = self.settings_state.read().process_motion_photos;
+        if enable {
+            self.enqueue(Box::new(move || {
+                sender.emit(PhotoExtractMotionTaskInput::Start)
+            }));
+        }
+    }
+
     fn add_task_photo_detect_faces(&mut self) {
-        let sender = self.photo_detect_faces.sender().clone();
+        let sender = self.photo_detect_faces_task.sender().clone();
         let mode = self.settings_state.read().face_detection_mode;
         match mode {
             FaceDetectionMode::Off => {}
             FaceDetectionMode::On => {
                 self.enqueue(Box::new(move || {
-                    sender.emit(PhotoDetectFacesInput::DetectForAllPictures)
+                    sender.emit(PhotoDetectFacesTaskInput::DetectForAllPictures)
                 }));
             }
         };
     }
 
     fn add_task_photo_detect_faces_for_one(&mut self, picture_id: PictureId) {
-        let sender = self.photo_detect_faces.sender().clone();
+        let sender = self.photo_detect_faces_task.sender().clone();
         let mode = self.settings_state.read().face_detection_mode;
         match mode {
             FaceDetectionMode::Off => {}
             FaceDetectionMode::On => {
                 self.enqueue(Box::new(move || {
-                    sender.emit(PhotoDetectFacesInput::DetectForOnePicture(picture_id))
+                    sender.emit(PhotoDetectFacesTaskInput::DetectForOnePicture(picture_id))
                 }));
             }
         };
     }
 
     fn add_task_photo_recognize_faces(&mut self) {
-        let sender = self.photo_recognize_faces.sender().clone();
+        let sender = self.photo_recognize_faces_task.sender().clone();
         let mode = self.settings_state.read().face_detection_mode;
         match mode {
             FaceDetectionMode::Off => {}
             FaceDetectionMode::On => {
                 self.enqueue(Box::new(move || {
-                    sender.emit(PhotoRecognizeFacesInput::Start)
+                    sender.emit(PhotoRecognizeFacesTaskInput::Start)
+                }));
+            }
+        };
+    }
+
+    fn add_task_person_thumbnails(&mut self) {
+        let sender = self.person_thumbnail_task.sender().clone();
+        let mode = self.settings_state.read().face_detection_mode;
+        match mode {
+            FaceDetectionMode::Off => {}
+            FaceDetectionMode::On => {
+                self.enqueue(Box::new(move || {
+                    sender.emit(PersonThumbnailTaskInput::Start)
                 }));
             }
         };
     }
 
     fn add_task_video_transcode(&mut self) {
-        let sender = self.video_transcode.sender().clone();
-        self.enqueue(Box::new(move || sender.emit(VideoTranscodeInput::Start)));
+        let sender = self.video_transcode_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(VideoTranscodeTaskInput::Start)));
     }
 
     fn add_task_load_library(&mut self, bootstrap_sender: Sender<BootstrapInput>) {
-        let sender = self.load_library.sender().clone();
+        let sender = self.load_library_task.sender().clone();
         let stale = self.library_stale.clone();
         let library_state = self.shared_state.clone();
         self.enqueue(Box::new(move || {
             if stale.load(Ordering::Relaxed) || library_state.read().is_empty() {
                 info!("Library stale or empty so refreshing.");
-                sender.emit(LoadLibraryInput::Refresh);
+                sender.emit(LoadLibraryTaskInput::Refresh);
             } else {
                 bootstrap_sender.emit(BootstrapInput::TaskCompleted(TaskName::LoadLibrary, None));
             }
         }));
     }
 
+    fn add_task_tidy(&mut self) {
+        let sender = self.tidy_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(TidyTaskInput::Start)));
+    }
+
+    fn add_task_migrate(&mut self) {
+        let sender = self.migrate_task.sender().clone();
+        self.enqueue(Box::new(move || sender.emit(MigrateTaskInput::Start)));
+    }
+
     fn enqueue(&mut self, task: Box<dyn Fn() + Send + Sync>) {
         if let Ok(mut vec) = self.pending_tasks.lock() {
             vec.push_back(task);
@@ -387,13 +439,13 @@ pub struct Bootstrap {
     controllers: Option<Controllers>,
 
     /// Current pictures base directory used by background tasks.
-    pictures_base_dir: Option<PathBuf>,
+    library_base_dir: Option<FlatpakPathBuf>,
 }
 
 impl Bootstrap {
     fn build_controllers(
         &mut self,
-        pic_base_dir: PathBuf,
+        library_base_dir: &FlatpakPathBuf,
         sender: &ComponentSender<Self>,
     ) -> anyhow::Result<Controllers> {
         let data_dir = glib::user_data_dir().join(APP_ID);
@@ -402,85 +454,96 @@ impl Bootstrap {
         let cache_dir = glib::user_cache_dir().join(APP_ID);
         let _ = std::fs::create_dir_all(&cache_dir);
 
-        let photo_scanner = photo::Scanner::build(&pic_base_dir)?;
+        // WARN duplicate thumbnail path calculation in app.rs
+        let thumbnail_dir = glib::user_cache_dir()
+            .join(APP_ID) // Remove to use standard XDG thumbnail path
+            .join("thumbnails");
 
-        let photo_repo =
-            photo::Repository::open(&pic_base_dir, &cache_dir, &data_dir, self.con.clone())?;
+        info!("Thumbnail directory is {:?}", thumbnail_dir);
 
-        let photo_thumbnailer = photo::Thumbnailer::build(&cache_dir)?;
+        let thumbnailer = Thumbnailer::build(&thumbnail_dir);
 
-        let video_scanner = video::Scanner::build(&pic_base_dir)?;
+        let photo_repo = photo::Repository::open(
+            library_base_dir,
+            &cache_dir,
+            &data_dir,
+            self.con.clone(),
+        )?;
 
-        let video_repo =
-            video::Repository::open(&pic_base_dir, &cache_dir, &data_dir, self.con.clone())?;
+        let photo_thumbnailer = photo::PhotoThumbnailer::build(thumbnailer.clone())?;
 
-        let video_thumbnailer = video::Thumbnailer::build(&cache_dir)?;
+        let scanner = Scanner::build(&library_base_dir.sandbox_path)?;
+
+        let video_repo = video::Repository::open(
+            library_base_dir,
+            &cache_dir,
+            &data_dir,
+            self.con.clone(),
+        )?;
+
+        let video_thumbnailer = video::VideoThumbnailer::build(thumbnailer.clone())?;
 
         let motion_photo_extractor = photo::MotionPhotoExtractor::build(&cache_dir)?;
 
-        let visual_repo = visual::Repository::open(&pic_base_dir, &cache_dir, self.con.clone())?;
+        let visual_repo = visual::Repository::open(
+            library_base_dir,
+            &cache_dir,
+            self.con.clone(),
+        )?;
 
-        let people_repo = people::Repository::open(&data_dir, self.con.clone())?;
+        let people_repo = people::Repository::open(
+            &cache_dir,
+            &data_dir,
+            self.con.clone())?;
 
         let stop = Arc::new(AtomicBool::new(false));
 
-        let load_library = LoadLibrary::builder()
+        let load_library_task = LoadLibraryTask::builder()
             .detach_worker((visual_repo.clone(), self.shared_state.clone()))
             .forward(sender.input_sender(), |msg| match msg {
-                LoadLibraryOutput::Done => {
+                LoadLibraryTaskOutput::Done => {
                     BootstrapInput::TaskCompleted(TaskName::LoadLibrary, None)
                 }
             });
 
-        let photo_scan = PhotoScan::builder()
-            .detach_worker((photo_scanner.clone(), photo_repo.clone()))
+        let library_scan_task = LibraryScanTask::builder()
+            .detach_worker((scanner, photo_repo.clone(), video_repo.clone()))
             .forward(sender.input_sender(), |msg| match msg {
-                PhotoScanOutput::Started => {
-                    BootstrapInput::TaskStarted(TaskName::Scan(MediaType::Photo))
+                LibraryScanTaskOutput::Started => {
+                    BootstrapInput::TaskStarted(TaskName::Scan)
                 }
-                PhotoScanOutput::Completed => {
-                    BootstrapInput::TaskCompleted(TaskName::Scan(MediaType::Photo), None)
+                LibraryScanTaskOutput::Completed => {
+                    BootstrapInput::TaskCompleted(TaskName::Scan, None)
                 }
             });
 
-        let video_scan = VideoScan::builder()
-            .detach_worker((video_scanner.clone(), video_repo.clone()))
-            .forward(sender.input_sender(), |msg| match msg {
-                VideoScanOutput::Started => {
-                    BootstrapInput::TaskStarted(TaskName::Scan(MediaType::Video))
-                }
-                VideoScanOutput::Completed => {
-                    BootstrapInput::TaskCompleted(TaskName::Scan(MediaType::Video), None)
-                }
-            });
-
-        let photo_enrich = PhotoEnrich::builder()
+        let photo_enrich_task = PhotoEnrichTask::builder()
             .detach_worker((stop.clone(), photo_repo.clone()))
             .forward(sender.input_sender(), |msg| match msg {
-                PhotoEnrichOutput::Started => {
+                PhotoEnrichTaskOutput::Started => {
                     BootstrapInput::TaskStarted(TaskName::Enrich(MediaType::Photo))
                 }
-                PhotoEnrichOutput::Completed(count) => {
+                PhotoEnrichTaskOutput::Completed(count) => {
                     BootstrapInput::TaskCompleted(TaskName::Enrich(MediaType::Photo), Some(count))
                 }
             });
 
-        let video_enrich = VideoEnrich::builder()
+        let video_enrich_task = VideoEnrichTask::builder()
             .detach_worker((
                 stop.clone(),
                 video_repo.clone(),
                 self.progress_monitor.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
-                VideoEnrichOutput::Started => {
+                VideoEnrichTaskOutput::Started => {
                     BootstrapInput::TaskStarted(TaskName::Enrich(MediaType::Video))
                 }
-                VideoEnrichOutput::Completed(count) => {
+                VideoEnrichTaskOutput::Completed(count) => {
                     BootstrapInput::TaskCompleted(TaskName::Enrich(MediaType::Video), Some(count))
                 }
             });
 
-        let photo_extract_motion = PhotoExtractMotion::builder()
+        let photo_extract_motion_task = PhotoExtractMotionTask::builder()
             .detach_worker((
                 stop.clone(),
                 motion_photo_extractor,
@@ -488,51 +551,53 @@ impl Bootstrap {
                 self.progress_monitor.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
-                PhotoExtractMotionOutput::Started => {
+                PhotoExtractMotionTaskOutput::Started => {
                     BootstrapInput::TaskStarted(TaskName::MotionPhoto)
                 }
-                PhotoExtractMotionOutput::Completed(count) => {
+                PhotoExtractMotionTaskOutput::Completed(count) => {
                     BootstrapInput::TaskCompleted(TaskName::MotionPhoto, Some(count))
                 }
             });
 
-        let photo_thumbnail = PhotoThumbnail::builder()
+        let photo_thumbnail_task = PhotoThumbnailTask::builder()
             .detach_worker((
                 stop.clone(),
+                thumbnail_dir.clone(),
                 photo_thumbnailer.clone(),
                 photo_repo.clone(),
                 self.progress_monitor.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
-                PhotoThumbnailOutput::Started => {
-                    BootstrapInput::TaskStarted(TaskName::Thumbnail(MediaType::Photo))
+                PhotoThumbnailTaskOutput::Started => {
+                    BootstrapInput::TaskStarted(TaskName::Thumbnail(ThumbnailType::Photo))
                 }
-                PhotoThumbnailOutput::Completed(count) => BootstrapInput::TaskCompleted(
-                    TaskName::Thumbnail(MediaType::Photo),
+                PhotoThumbnailTaskOutput::Completed(count) => BootstrapInput::TaskCompleted(
+                    TaskName::Thumbnail(ThumbnailType::Photo),
                     Some(count),
                 ),
             });
 
-        let video_thumbnail = VideoThumbnail::builder()
+        let video_thumbnail_task = VideoThumbnailTask::builder()
             .detach_worker((
                 stop.clone(),
+                thumbnail_dir.clone(),
                 video_thumbnailer.clone(),
                 video_repo.clone(),
                 self.progress_monitor.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
-                VideoThumbnailOutput::Started => {
-                    BootstrapInput::TaskStarted(TaskName::Thumbnail(MediaType::Video))
+                VideoThumbnailTaskOutput::Started => {
+                    BootstrapInput::TaskStarted(TaskName::Thumbnail(ThumbnailType::Video))
                 }
-                VideoThumbnailOutput::Completed(count) => BootstrapInput::TaskCompleted(
-                    TaskName::Thumbnail(MediaType::Video),
+                VideoThumbnailTaskOutput::Completed(count) => BootstrapInput::TaskCompleted(
+                    TaskName::Thumbnail(ThumbnailType::Video),
                     Some(count),
                 ),
             });
 
         let transcoder = video::Transcoder::new(&cache_dir);
 
-        let video_transcode = VideoTranscode::builder()
+        let video_transcode_task = VideoTranscodeTask::builder()
             .detach_worker((
                 stop.clone(),
                 self.shared_state.clone(),
@@ -541,52 +606,53 @@ impl Bootstrap {
                 self.progress_monitor.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
-                VideoTranscodeOutput::Started => BootstrapInput::TaskStarted(TaskName::Transcode),
-                VideoTranscodeOutput::Completed => {
+                VideoTranscodeTaskOutput::Started => BootstrapInput::TaskStarted(TaskName::Transcode),
+                VideoTranscodeTaskOutput::Completed => {
                     BootstrapInput::TaskCompleted(TaskName::Transcode, None)
                 }
             });
 
-        let photo_clean = PhotoClean::builder()
+        let photo_clean_task = PhotoCleanTask::builder()
             .detach_worker((stop.clone(), photo_repo.clone()))
             .forward(sender.input_sender(), |msg| match msg {
-                PhotoCleanOutput::Started => {
+                PhotoCleanTaskOutput::Started => {
                     BootstrapInput::TaskStarted(TaskName::Clean(MediaType::Photo))
                 }
-                PhotoCleanOutput::Completed(count) => {
+                PhotoCleanTaskOutput::Completed(count) => {
                     BootstrapInput::TaskCompleted(TaskName::Clean(MediaType::Photo), Some(count))
                 }
             });
 
-        let video_clean = VideoClean::builder()
+        let video_clean_task = VideoCleanTask::builder()
             .detach_worker((stop.clone(), video_repo.clone()))
             .forward(sender.input_sender(), |msg| match msg {
-                VideoCleanOutput::Started => {
+                VideoCleanTaskOutput::Started => {
                     BootstrapInput::TaskStarted(TaskName::Clean(MediaType::Video))
                 }
-                VideoCleanOutput::Completed(count) => {
+                VideoCleanTaskOutput::Completed(count) => {
                     BootstrapInput::TaskCompleted(TaskName::Clean(MediaType::Video), Some(count))
                 }
             });
 
-        let photo_detect_faces = PhotoDetectFaces::builder()
+        let photo_detect_faces_task = PhotoDetectFacesTask::builder()
             .detach_worker((
                 stop.clone(),
-                data_dir,
+                data_dir.clone(),
+                thumbnailer.clone(),
                 photo_repo.clone(),
                 people_repo.clone(),
                 self.progress_monitor.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
-                PhotoDetectFacesOutput::Started => {
+                PhotoDetectFacesTaskOutput::Started => {
                     BootstrapInput::TaskStarted(TaskName::DetectFaces)
                 }
-                PhotoDetectFacesOutput::Completed => {
+                PhotoDetectFacesTaskOutput::Completed => {
                     BootstrapInput::TaskCompleted(TaskName::DetectFaces, None)
                 }
             });
 
-        let photo_recognize_faces = PhotoRecognizeFaces::builder()
+        let photo_recognize_faces_task = PhotoRecognizeFacesTask::builder()
             .detach_worker((
                 stop.clone(),
                 cache_dir.clone(),
@@ -594,32 +660,71 @@ impl Bootstrap {
                 self.progress_monitor.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
-                PhotoRecognizeFacesOutput::Started => {
+                PhotoRecognizeFacesTaskOutput::Started => {
                     BootstrapInput::TaskStarted(TaskName::RecognizeFaces)
                 }
-                PhotoRecognizeFacesOutput::Completed => {
+                PhotoRecognizeFacesTaskOutput::Completed => {
                     BootstrapInput::TaskCompleted(TaskName::RecognizeFaces, None)
                 }
             });
 
+        let tidy_task = TidyTask::builder()
+            .detach_worker(stop.clone())
+            .forward(sender.input_sender(), |msg| match msg {
+                TidyTaskOutput::Started => {
+                    BootstrapInput::TaskStarted(TaskName::Tidy)
+                }
+                TidyTaskOutput::Completed => {
+                    BootstrapInput::TaskCompleted(TaskName::Tidy, None)
+                }
+            });
+
+        let migrate = Migrate::build(people_repo, &data_dir, library_base_dir.clone());
+
+        let migrate_task = MigrateTask::builder()
+            .detach_worker((stop.clone(), migrate))
+            .forward(sender.input_sender(), |msg| match msg {
+                MigrateTaskOutput::Started => {
+                    BootstrapInput::TaskStarted(TaskName::Migrate)
+                }
+                MigrateTaskOutput::Completed => {
+                    BootstrapInput::TaskCompleted(TaskName::Migrate, None)
+                }
+            });
+
+        let person_thumbnailer = people::PersonThumbnailer::build(thumbnailer.clone(), &cache_dir);
+
+        let person_thumbnail_task = PersonThumbnailTask::builder()
+            .detach_worker((stop.clone(), person_thumbnailer, photo_repo.clone(), self.progress_monitor.clone()))
+            .forward(sender.input_sender(), |msg| match msg {
+                PersonThumbnailTaskOutput::Started => {
+                    BootstrapInput::TaskStarted(TaskName::Thumbnail(ThumbnailType::Face))
+                }
+                PersonThumbnailTaskOutput::Completed(count) => {
+                    BootstrapInput::TaskCompleted(TaskName::Thumbnail(ThumbnailType::Face), Some(count))
+                }
+            });
+
         let mut controllers = Controllers {
             stop,
             started_at: None,
             shared_state: self.shared_state.clone(),
             settings_state: self.settings_state.clone(),
-            load_library: Arc::new(load_library),
-            photo_scan: Arc::new(photo_scan),
-            video_scan: Arc::new(video_scan),
-            photo_enrich: Arc::new(photo_enrich),
-            video_enrich: Arc::new(video_enrich),
-            photo_extract_motion: Arc::new(photo_extract_motion),
-            photo_clean: Arc::new(photo_clean),
-            video_clean: Arc::new(video_clean),
-            photo_thumbnail: Arc::new(photo_thumbnail),
-            video_thumbnail: Arc::new(video_thumbnail),
-            photo_detect_faces: Arc::new(photo_detect_faces),
-            photo_recognize_faces: Arc::new(photo_recognize_faces),
-            video_transcode: Arc::new(video_transcode),
+            load_library_task: Arc::new(load_library_task),
+            library_scan_task: Arc::new(library_scan_task),
+            photo_enrich_task: Arc::new(photo_enrich_task),
+            video_enrich_task: Arc::new(video_enrich_task),
+            photo_extract_motion_task: Arc::new(photo_extract_motion_task),
+            photo_clean_task: Arc::new(photo_clean_task),
+            video_clean_task: Arc::new(video_clean_task),
+            photo_thumbnail_task: Arc::new(photo_thumbnail_task),
+            video_thumbnail_task: Arc::new(video_thumbnail_task),
+            photo_detect_faces_task: Arc::new(photo_detect_faces_task),
+            photo_recognize_faces_task: Arc::new(photo_recognize_faces_task),
+            video_transcode_task: Arc::new(video_transcode_task),
+            tidy_task: Arc::new(tidy_task),
+            migrate_task: Arc::new(migrate_task),
+            person_thumbnail_task: Arc::new(person_thumbnail_task),
             pending_tasks: Arc::new(Mutex::new(VecDeque::new())),
             is_running: false,
             library_stale: Arc::new(AtomicBool::new(true)),
@@ -629,8 +734,10 @@ impl Bootstrap {
 
         // Initial library load to reduce time from starting app and seeing a photo grid
         controllers.add_task_load_library(sender.input_sender().clone());
-        controllers.add_task_photo_scan();
-        controllers.add_task_video_scan();
+
+        controllers.add_task_migrate();
+
+        controllers.add_task_library_scan();
         controllers.add_task_photo_enrich();
         controllers.add_task_video_enrich();
 
@@ -647,10 +754,12 @@ impl Bootstrap {
         controllers.add_task_photo_detect_faces();
         controllers.add_task_photo_recognize_faces();
 
-        // This is the last background task to complete. Refresh library if there
-        // has been a visible change to the library state.
+        controllers.add_task_tidy();
+
         controllers.add_task_load_library(sender.input_sender().clone());
 
+        controllers.add_task_person_thumbnails();
+
         Ok(controllers)
     }
 }
@@ -679,7 +788,7 @@ impl Worker for Bootstrap {
             progress_monitor,
             con,
             controllers: None,
-            pictures_base_dir: None,
+            library_base_dir: None,
         }
     }
 
@@ -687,14 +796,18 @@ impl Worker for Bootstrap {
         // This match block coordinates the background tasks launched immediately after
         // the app starts up.
         match msg {
-            BootstrapInput::Configure(pictures_base_dir) => {
+            BootstrapInput::Configure(library_base_dir) => {
                 info!(
                     "Configuring with pictures base directory: {:?}",
-                    pictures_base_dir
+                    library_base_dir
                 );
-                match self.build_controllers(pictures_base_dir.clone(), &sender) {
+
+                match self.build_controllers(
+                    &library_base_dir,
+                    &sender,
+                ) {
                     Ok(controllers) => {
-                        self.pictures_base_dir = Some(pictures_base_dir);
+                        self.library_base_dir = Some(library_base_dir);
                         self.controllers = Some(controllers);
                         sender.input(BootstrapInput::Start);
                     }
@@ -707,9 +820,9 @@ impl Worker for Bootstrap {
                 info!("Settings updated.");
                 // Only stop, reconfigure, and restart tasks if pictures dir changes.
                 if self
-                    .pictures_base_dir
+                    .library_base_dir
                     .as_ref()
-                    .is_some_and(|dir| *dir != settings.pictures_base_dir)
+                    .is_some_and(|dir| *dir != settings.library_base_dir)
                 {
                     // If running, then shutdown running and queued tasks, and then reconfigure.
                     // Otherwise simply reconfigure with new path.
@@ -718,21 +831,24 @@ impl Worker for Bootstrap {
                         .as_ref()
                         .is_some_and(|controllers| controllers.is_running)
                     {
-                        self.pictures_base_dir = None;
+                        self.library_base_dir = None;
                         sender.input(BootstrapInput::Stop);
                     } else {
                         self.controllers = None;
-                        sender.input(BootstrapInput::Configure(settings.pictures_base_dir));
+                        sender.input(BootstrapInput::Configure(
+                            settings.library_base_dir.clone(),
+                        ));
                     }
                 }
             }
-            BootstrapInput::Stopped if self.pictures_base_dir.is_none() => {
+            BootstrapInput::Stopped if self.library_base_dir.is_none() => {
                 // If stopped and no pictures base dir, then background tasks were
                 // shutdown in response to the user changing the pictures base directory.
                 // Now that tasks are shutdown, it is safe to reconfigure with
                 // the new directory.
+                let settings = self.settings_state.read();
                 sender.input(BootstrapInput::Configure(
-                    self.settings_state.read().pictures_base_dir.clone(),
+                    settings.library_base_dir.clone(),
                 ));
             }
             BootstrapInput::TaskCompleted(TaskName::LoadLibrary, _)
